/*
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of the Example Code Package for BASE.
	Available at http://baseplugins.thep.lu.se/
	BASE main site: http://base.thep.lu.se/
	
	This is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	The software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.examples.extensions.list;


import java.util.Iterator;

import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.extensions.list.AbstractListColumnBean;
import net.sf.basedb.clients.web.extensions.list.ListColumnAction;
import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.util.extensions.InvokationContext;
import net.sf.basedb.util.formatter.Formatter;

/**
	Adds support for a list column that display data files attached to
	an item. This factory can be used for all items that implements
	the {@link FileStoreEnabled} interface. The added column will display
	all file members in the file set together with links for
	editing, viewing and downloading the file. The column is filterable
	and exportable (without links), but it is not possible to sort.
	
	@author nicklas
	@since 1.5
*/
public class DataFilesActionFactory 
	extends AbstractJspActionFactory<ListColumnAction<FileStoreEnabled, String>>

{

	public DataFilesActionFactory()
	{}

	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@SuppressWarnings("unchecked")
	@Override
	public ListColumnAction<FileStoreEnabled, String>[] getActions(InvokationContext<? super ListColumnAction<FileStoreEnabled, String>> context)
	{
		JspContext jspContext = (JspContext)context.getClientContext();
		String sessionId = jspContext.getSessionControl().getId();
		String rootUrl = jspContext.getRoot() + "/";
		return new ListColumnAction[] { new DataFilesAction(sessionId, rootUrl) };
	}
	// ----------------------------------
	
	/**
		This is the action that adds the column to the table. 		
	*/
	static class DataFilesAction
		extends AbstractListColumnBean<FileStoreEnabled, String>
	{

		private final String sessionId;
		private final String rootUrl;
		private final ItemQuery<FileSetMember> query;
		/**
			Create a new action. The session ID and root url
			are needed to create the view and download file 
			linkes.
		*/
		DataFilesAction(String sessionId, String rootUrl)
		{
			this.sessionId = sessionId;
			this.rootUrl = rootUrl;
			query = FileSetMember.getQuery(null);
			query.order(Orders.asc(Hql.property("dataFileType.name")));
			setId("x-data-files");
			setTitle("Data files");
			setProperty("&fileSet.members(file.name)");
			setExportable(true);
			setFilterable(true);
			setValueType(Type.STRING);
		}
		
		/*
			From the ListColumnAction interface
			-----------------------------------
		*/
		/**
			Generates a comma-separated list with file names. Uses
			the {@link Base#getLinkedFile(String, File, boolean, boolean, boolean, String)}
			to create links for editing, viewing and downloading the file.
		*/
		@Override
		public String getValue(DbControl dc, FileStoreEnabled item)
		{
			StringBuilder sb = new StringBuilder();
			if (item.hasFileSet())
			{
				query.setEntityParameter("fileSet", item.getFileSet());
				Iterator<FileSetMember> it = query.iterate(dc);
				
				while (it.hasNext())
				{
					FileSetMember member = it.next();
					File f = member.getFile();
					DataFileType fileType = member.getDataFileType();
					
					if (sb.length() > 0) sb.append(", ");
					sb.append(HTML.encodeTags(fileType.getName())).append(": ");
					sb.append(Base.getLinkedFile(sessionId, f, false, true, true, rootUrl));
				}
			}
			return sb.toString();
		}

		/**
			When exporting, we simple return the list of file set members
			and let the {@link FileSetMemberFormatter} generate the output.
		*/
		@Override
		public Object getExportValue(DbControl dc, FileStoreEnabled item) 
		{
			if (item.hasFileSet())
			{
				query.setEntityParameter("fileSet", item.getFileSet());
				return query.list(dc);
			}
			return null;
		}
		@Override
		public Formatter<?> getExportFormatter() 
		{
			return new FileSetMemberFormatter();
		}
		// ----------------------------------------
	}
	
	/**
		This formatter is used to generate the output when exporting to a file. It
		will display: 'FileType: path' for each file.
	*/
	static class FileSetMemberFormatter
		implements Formatter<FileSetMember>
	{

		FileSetMemberFormatter()
		{}
		
		@Override
		public String format(FileSetMember member) 
		{
			File file = member.getFile();
			DataFileType fileType = member.getDataFileType();
			return fileType.getName() + ": " + file.getPath().toString();
		}

		@Override
		public FileSetMember parseString(String value) 
		{
			return null;
		}
		
	}
}
